#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Aug  4 14:50:47 2021

@author: vkapoor
"""
from tifffile import imread
import csv
import napari
import glob
import os

import numpy as np
import json
import pandas as pd
from pathlib import Path
from scipy import spatial

from qtpy.QtCore import Qt

from oneat.NEATUtils.oneat_animation.OneatVisualization import MidSlices
from oneat.NEATUtils.oneat_animation._qt import OneatWidget


default_reader = 'tifffile'


class NEATViz(object):

        def __init__(self, imagedir,   
                        csvdir,
                        savedir, 
                        categories_json, 
                        imagereader = default_reader , 
                        heatmapimagedir = None, 
                        segimagedir = None, 
                        heatname = '_Heat', 
                        eventname = '_Event', 
                        fileextension = '*tif', 
                        blur_radius = 5, 
                        start_project_mid = 0, 
                        end_project_mid = 0,
                        headless = False,
                        event_threshold = 0.999,
                        nms_space = 10,
                        nms_time = 3
                         ):
            
            
               self.imagedir = imagedir
               self.heatmapimagedir = heatmapimagedir
               self.segimagedir = segimagedir
               self.savedir = savedir
               self.csvdir = csvdir
               self.heatname = heatname
               self.eventname = eventname
               self.headless = headless 
               self.event_threshold = event_threshold
               self.nms_space = nms_space
               self.nms_time = nms_time
               self.categories_json = categories_json
               self.start_project_mid = start_project_mid
               self.end_project_mid = end_project_mid
               self.fileextension = fileextension
               self.blur_radius = blur_radius
               self.imagereader = imagereader
               if self.imagereader == default_reader:
                   self.use_dask = False
               else:
                   self.use_dask = True    
               Path(self.savedir).mkdir(exist_ok=True)
               Path(self.csvdir).mkdir(exist_ok=True)
               self.viewer = napari.Viewer()
               
               self.time = 0
               self.load_json()
               self.key_categories = self.load_json()
               
               
               if self.headless == False:
                  self.showNapari()
               if self.headless:
                   self.donotshowNapari()   
               
        def load_json(self):
            with open(self.categories_json, 'r') as f:
                return json.load(f)      
            
            
        def donotshowNapari(self):
                 
                 
                 Raw_path = os.path.join(self.imagedir, self.fileextension)
                 X = glob.glob(Raw_path)
                 
                 
                 
                 for imagename in X:
                         
                        Name  = os.path.basename(os.path.splitext(imagename)[0]) 
                        image = imread(imagename)
                        seg_image = imread(self.segimagedir + Name + '.tif')
                        if len(seg_image.shape) == 4:
                          seg_image =  MidSlices(seg_image, self.start_project_mid, self.end_project_mid, False, axis = 1)
                        headlesscall(image, imagename, self.key_categories, self.event_threshold, self.nms_space, self.nms_time, self.csvdir, self.savedir)     
                               
                                
                 
        def showNapari(self):
                 
                 self.oneat_widget = OneatWidget(self.viewer, self.csvdir, self.savedir, 'Name', 
                 self.key_categories, use_dask = self.use_dask, segimagedir = self.segimagedir,
                 heatimagedir = self.heatmapimagedir, heatname = self.heatname, 
                 start_project_mid = self.start_project_mid,
                 end_project_mid = self.end_project_mid )
                 Raw_path = os.path.join(self.imagedir, self.fileextension)
                 X = glob.glob(Raw_path)
                 Imageids = []
                 self.oneat_widget.frameWidget.imageidbox.addItem('Select Image')
                 self.oneat_widget.frameWidget.eventidbox.addItem('Select Event')
                 for imagename in X:
                     Imageids.append(imagename)

                 for i in range(0, len(Imageids)):
                     self.oneat_widget.frameWidget.imageidbox.addItem(str(Imageids[i]))
                 
                 for (event_name,event_label) in self.key_categories.items():
                     if event_label > 0:
                         self.oneat_widget.frameWidget.eventidbox.addItem(event_name)

                 dock_widget = self.viewer.window.add_dock_widget(self.oneat_widget, area='right')
                 self.viewer.window._qt_window.resizeDocks([dock_widget], [200], Qt.Horizontal)  

                 napari.run()

def cluster_points(event_locations_dict,event_locations_size_dict, nms_space, nms_time):

     for (k,v) in event_locations_dict.items():
         currenttime = k
         event_locations = v
       
        
         tree = spatial.cKDTree(event_locations)
         for i in range(1, nms_time):
                    
                    forwardtime = currenttime + i
                    if int(forwardtime) in event_locations_dict.keys():
                      forward_event_locations = event_locations_dict[int(forwardtime)]
                      for location in forward_event_locations:
                        if (int(forwardtime), int(location[0]), int(location[1])) in event_locations_size_dict:   
                                forwardsize, forwardscore = event_locations_size_dict[int(forwardtime), int(location[0]), int(location[1])]
                                distance, nearest_location = tree.query(location)
                                nearest_location = int(event_locations[nearest_location][0]), int(event_locations[nearest_location][1])

                                if distance <= nms_space:
                                            if (int(currenttime), int(nearest_location[0]), int(nearest_location[1])) in event_locations_size_dict:
                                                currentsize, currentscore = event_locations_size_dict[int(currenttime), int(nearest_location[0]), int(nearest_location[1])]
                                                if  currentsize >= forwardsize:
                                                    event_locations_size_dict.pop((int(forwardtime), int(location[0]), int(location[1])))
                                                    
                                                if currentsize < forwardsize:
                                                    event_locations_size_dict.pop((int(currenttime), int(nearest_location[0]), int(nearest_location[1])))   
     return event_locations_size_dict                                                     

def headlesscall(image, imagename, key_categories, event_threshold, nms_space, nms_time,csvdir, savedir):
            for (event_name,event_label) in key_categories.items():
                            if event_label > 0:
                                event_locations = []
                                size_locations = []
                                score_locations = []
                                event_locations = []
                                confidence_locations = []
                                event_locations_dict = {}
                                event_locations_size_dict = {}
                                csvname = csvdir + "/" + event_name + "Location" + (os.path.splitext(os.path.basename(imagename))[0] + '.csv')
                                dataset   = pd.read_csv(csvname, delimiter = ',')
                                dataset_index =  dataset.index
                                #Data is written as T, Y, X, Score, Size, Confidence
                                T =  dataset[ dataset.keys()[0]][0:]
                                Z =  dataset[ dataset.keys()[1]][0:]
                                Y = dataset[dataset.keys()[2]][0:]
                                X = dataset[dataset.keys()[3]][0:]
                                Score = dataset[dataset.keys()[4]][0:]
                                Size = dataset[dataset.keys()[5]][0:]
                                Confidence = dataset[dataset.keys()[6]][0:]
                                listtime = T.tolist()
                                listz = Z.tolist()
                                listy = Y.tolist()
                                listx = X.tolist()
                                listsize = Size.tolist()
                                
                                
                                listscore = Score.tolist()
                                listconfidence = Confidence.tolist()
                                
                            
                                for i in (range(len(listtime))):
                                        
                                        tcenter = int(listtime[i])
                                        zcenter = listz[i]
                                        ycenter = listy[i]
                                        xcenter = listx[i]
                                        size = listsize[i]
                                        score = listscore[i]
                                        confidence = listconfidence[i]   
                                        if score > event_threshold:
                                                event_locations.append([int(tcenter), int(ycenter), int(xcenter)])   

                                                if int(tcenter) in event_locations_dict.keys():
                                                    current_list = event_locations_dict[int(tcenter)]
                                                    current_list.append([int(ycenter), int(xcenter)])
                                                    event_locations_dict[int(tcenter)] = current_list 
                                                    event_locations_size_dict[(int(tcenter), int(ycenter), int(xcenter))] = [size, score]
                                                else:
                                                    current_list = []
                                                    current_list.append([int(ycenter), int(xcenter)])
                                                    event_locations_dict[int(tcenter)] = current_list    
                                                    event_locations_size_dict[int(tcenter), int(ycenter), int(xcenter)] = [size, score]

                                                size_locations.append(size)
                                                score_locations.append(score)
                                                confidence_locations.append(confidence)

                                event_locations_size_dict = cluster_points(event_locations_dict,event_locations_size_dict, nms_space, nms_time)
                                event_locations_clean = []             
                                dict_locations = event_locations_size_dict.keys()
                                tlocations = []
                                zlocations = []   
                                ylocations = []
                                xlocations = []
                                scores = []
                                radiuses = []
                                confidences = []
                                angles = []
                                for location, sizescore in event_locations_size_dict.items():
                                    tlocations.append(float(location[0]))
                                    if len(image.shape) == 4:
                                        zlocations.append(float(image.shape[1]//2))
                                    else:
                                        zlocations.append(0)
                                    ylocations.append(float(location[1]))
                                    xlocations.append(float(location[2]))
                                    scores.append(float(sizescore[1])) 
                                    radiuses.append(float(sizescore[0]))
                                    confidences.append(1)
                                    angles.append(2)
                                for location  in dict_locations:
                                    event_locations_clean.append(location)
                                        

                                event_count = np.column_stack(
                                            [tlocations, zlocations, ylocations, xlocations, scores, radiuses, confidences, angles])
                                event_count = sorted(event_count, key=lambda x: x[0], reverse=False)
                                
                                event_data = []
                                csvname = savedir + "/" + 'Clean' +  event_name + "Location" + (
                                os.path.splitext(os.path.basename(imagename))[0])
                                if(os.path.exists(csvname + ".csv")):
                                            os.remove(csvname + ".csv")
                                writer = csv.writer(open(csvname + ".csv", "a", newline=''))
                                filesize = os.stat(csvname + ".csv").st_size

                                if filesize < 1:
                                            writer.writerow(['T', 'Z', 'Y', 'X', 'Score', 'Size', 'Confidence', 'Angle'])
                                for line in event_count:
                                            if line not in event_data:
                                                event_data.append(line)
                                            writer.writerows(event_data)
                                            event_data = []        


from .NEATModels import *
from .NEATUtils import *
from ._version import __version__
from csbdeep.utils.tf import keras_import
from . import NEATDynamic, NEATSynamic

from oneat.pretrained import register_model, register_aliases, clear_models_and_aliases

get_file = keras_import('utils', 'get_file')


clear_models_and_aliases(NEATDynamic, NEATSynamic)

register_model(NEATSynamic,   'Cellsplitdetectorbrightfield',  'https://zenodo.org/record/6481021/files/Cellsplitdetectorbrightfield.h5', '0c6ba49c1ba0eb91819af40460fb66cb',
                               'Cellsplitcordhelaflou'         ,   'https://zenodo.org/record/6481021/files/Cellsplitcordhelaflou.json', 'aed21cb69d6fb8be32c47f78a39d32f5',
                                'Cellsplitcategorieshelaflou'  , 'https://zenodo.org/record/6481021/files/Cellsplitcategorieshelaflou.json', '7a67a83f08fb1add3c1b1a3e0eeec773',
                                 'Cellsplitdetectorbrightfield_Parameter' , 'https://zenodo.org/record/6481021/files/Cellsplitdetectorbrightfield_Parameter.json', '266e3e7fa7587d53d62ae0492482a1bc'  )  



register_model(NEATSynamic,   'Cellsplitdetectorhdpc',  'https://zenodo.org/record/6483483/files/Cellsplitdetectorhdpc.h5', '701d7fc8494b66a73b024f3d3c3d3dad',
                               'Cellsplitcordhelaflou'         ,   'https://zenodo.org/record/6481021/files/Cellsplitcordhelaflou.json', 'aed21cb69d6fb8be32c47f78a39d32f5',
                                'Cellsplitcategorieshelaflou'  , 'https://zenodo.org/record/6481021/files/Cellsplitcategorieshelaflou.json', '7a67a83f08fb1add3c1b1a3e0eeec773',
                                 'Cellsplitdetectorhdpc_Parameter' , 'https://zenodo.org/record/6483483/files/Cellsplitdetectorhdpc_Parameter.json', '556f0ce063a4cacda6b9f27ae7aae69b'  )  

register_model(NEATSynamic,   'Cellsplitdetectorxenopus',  'https://zenodo.org/record/6484966/files/Cellsplitdetectorxenopus.h5', '299a2edebf217da76732fd812cb5d6fe',
                               'Cellsplitcordxenopus'         ,   'https://zenodo.org/record/6484966/files/Cellsplitcordxenopus.json', 'aed21cb69d6fb8be32c47f78a39d32f5',
                                'Cellsplitcategoriesxenopus'  , 'https://zenodo.org/record/6484966/files/Cellsplitcategoriesxenopus.json', '7a67a83f08fb1add3c1b1a3e0eeec773',
                                 'Cellsplitdetectorxenopus_Parameter' , 'https://zenodo.org/record/6484966/files/Cellsplitdetectorxenopus_Parameter.json', 'ee3dfb4ff1af80e44ceaebbdf4b1bff1'  )                                  

register_aliases(NEATSynamic, 'Cellsplitdetectorbrightfield',  'Cellsplitdetectorbrightfield')
register_aliases(NEATSynamic, 'Cellsplitdetectorhdpc',  'Cellsplitdetectorhdpc')
register_aliases(NEATSynamic, 'Cellsplitdetectorxenopus',  'Cellsplitdetectorxenopus')


del register_model, register_aliases, clear_models_and_aliases


def abspath(path):
    import os
    base_path = os.path.abspath(os.path.dirname(__file__))
    return os.path.join(base_path, path)



def test_image_brightfield(target):
    from tifffile import imread, imwrite
    import os
    url = "https://zenodo.org/record/6480142/files/20210904_TL2%20-%20R05-C03-F0_ch_2.tif"
    hash = "67e13fa4df301dfe2c2a57f785aedada"
    fname = abspath(get_file(fname='brightfield', origin=url, file_hash=hash))
    image = imread(fname)
    Name = os.path.basename(os.path.splitext(fname)[0])
    imwrite(target + '/' + Name + '.tif', image.astype('float32'))
    return fname  

def test_image_hdpc(target):
    from tifffile import imread, imwrite
    import os
    url = "https://zenodo.org/record/6483483/files/20210904_TL2%20-%20R05-C03-F0_ch_1.tif"
    hash = "ec6fde9a27a627866e88d44a6d98f41e"
    fname = abspath(get_file(fname='hdpc', origin=url, file_hash=hash))
    image = imread(fname)
    Name = os.path.basename(os.path.splitext(fname)[0])
    imwrite(target + '/' + Name + '.tif', image.astype('float32'))
    return fname  

def test_image_xenopus(target):
    from tifffile import imread, imwrite
    import os
    url = "https://zenodo.org/record/6484966/files/C1-for_oneat_prediction.tif"
    hash = "ccf1836e66c350f90a3f458890d16220"
    fname = abspath(get_file(fname='xenopus', origin=url, file_hash=hash))
    image = imread(fname)
    Name = os.path.basename(os.path.splitext(fname)[0])
    imwrite(target + '/' + Name + '.tif', image.astype('float32'))
    return fname     



import csv
import numpy as np
from tifffile import imread, imwrite 
import pandas as pd
import os
from tqdm import tqdm
import glob
import json
from skimage import measure
from pathlib import Path
from sklearn.model_selection import train_test_split
from .helpers import  normalizeFloatZeroOne
from PIL import Image
from dask.array.image import imread as daskread

"""
@author: Varun Kapoor
In this program we create training movies and training images for ONEAT. The training data comprises of images and text labels attached to them.
TrainingMovies: This program is for action recognition training data creation. The inputs are the training image, the corresponding integer labelled segmentation image,
csv file containing time, ylocation, xlocation, angle (optional)
Additional parameters to be supplied are the 
1) sizeTminus: action events are centered at the time location, this parameter is the start time of the time volume the network carved out from the image.
2) sizeTplus: this parameter is the end of the time volume to be carved out from the image.
3) total_categories: It is the number of total action categories the network is supposed to predict, Vanilla ONEAT has these labels:
   0: NormalEvent
   1: ApoptosisEvent
   2: DivisionEvent
   3: Macrocheate as static dynamic event
   4: Non MatureP1 cells as static dynamic event
   5: MatureP1 cells as static dynamic event
    
TrainingImages: This program is for cell type recognition training data creation. The inputs are the trainng image, the corresponding integer labelled segmentation image,
Total categories for cell classification part of vanilla ONEAT are:
    0: Normal cells
    1: Central time frame of apoptotic cell
    2: Central time frame of dividing cell
    3: Macrocheates
    4: Non MatureP1 cells
    5: MatureP1 cells
csv file containing time, ylocation, xlocation of that event/cell type
"""    
   
def SegFreeMovieLabelDataSet(image_dir, csv_dir, save_dir, static_name, static_label, csv_name_diff, crop_size,gridx = 1, gridy = 1, normPatch = False,  yolo_v1 = True, yolo_v2 = False, tshift = 0, normalizeimage = True):
    
    
            raw_path = os.path.join(image_dir, '*tif')
            Csv_path = os.path.join(csv_dir, '*csv')
            files_raw = glob.glob(raw_path)
            files_raw.sort
            filesCsv = glob.glob(Csv_path)
            filesCsv.sort
            Path(save_dir).mkdir(exist_ok=True)
            total_categories = len(static_name)
                
            
            
            for fname in files_raw:
                          
                         name = os.path.basename(os.path.splitext(fname)[0])  
                         
                        
                         for csvfname in filesCsv:
                                 count = 0  
                                 Csvname =  os.path.basename(os.path.splitext(csvfname)[0])
                                 
                                 for i in  range(0, len(static_name)):
                                     event_name = static_name[i]
                                     trainlabel = static_label[i]
                                     classfound = (Csvname == csv_name_diff +  event_name+ name )  
                                     if classfound:
                                                    print(Csvname)
                                                    image = imread(fname)
                                                    if normPatch ==False and normalizeimage == True:
                                                       image = normalizeFloatZeroOne( image.astype('uint16'),1,99.8)
                                                    dataset = pd.read_csv(csvfname)
                                                    time = dataset[dataset.keys()[0]][1:]
                                                    y = dataset[dataset.keys()[1]][1:]
                                                    x = dataset[dataset.keys()[2]][1:]
                                                                                
                                                                             
                                                    #Categories + XYHW + Confidence 
                                                    for (key, t) in time.items():
                                                    
                                                          SimpleMovieMaker(t, y[key], x[key], image, crop_size,gridx, gridy, total_categories, trainlabel, name+ event_name + str(count), save_dir, normPatch,yolo_v1, yolo_v2, tshift) 
                                                          count = count + 1
                                                        
                                                      
                                                        
def SegFreeMovieLabelDataSet4D(image_dir, csv_dir, save_dir, static_name, static_label, csv_name_diff, crop_size, gridx = 1, gridy = 1, normPatch = False,  yolo_v1 = True, yolo_v2 = False, tshift = 0, normalizeimage = True):
    
    
            raw_path = os.path.join(image_dir, '*tif')
            Csv_path = os.path.join(csv_dir, '*csv')
            files_raw = glob.glob(raw_path)
            files_raw.sort
            filesCsv = glob.glob(Csv_path)
            filesCsv.sort
            Path(save_dir).mkdir(exist_ok=True)
            total_categories = len(static_name)
                
            
            
            for fname in files_raw:
                          
                         name = os.path.basename(os.path.splitext(fname)[0])  
                         
                        
                         for csvfname in filesCsv:
                                 count = 0  
                                 Csvname =  os.path.basename(os.path.splitext(csvfname)[0])
                                 
                                 for i in  range(0, len(static_name)):
                                     event_name = static_name[i]
                                     trainlabel = static_label[i]
                                     classfound = (Csvname == csv_name_diff +  event_name+ name )  
                                     if classfound:
                                                    print(Csvname)
                                                    image = imread(fname)
                                                    dataset = pd.read_csv(csvfname)
                                                    time = dataset[dataset.keys()[0]][1:]
                                                    z = dataset[dataset.keys()[1]][1:]
                                                    y = dataset[dataset.keys()[2]][1:]
                                                    x = dataset[dataset.keys()[3]][1:]
                                                                                
                                                                             
                                                    #Categories + XYHW + Confidence 
                                                    for (key, t) in time.items():
                                                       try: 
                                                          SimpleMovieMaker4D(normalizeimage, t, z[key], y[key], x[key], image, crop_size,gridx, gridy, total_categories, trainlabel, name+ event_name + str(count), save_dir, normPatch, yolo_v1, yolo_v2, tshift) 
                                                          count = count + 1
                                                        
                                                       except:
                                                        
                                                           pass
def SimpleMovieMaker(time, y, x, image, crop_size,gridx, gridy, total_categories, trainlabel, name, save_dir, normPatch,yolo_v1, yolo_v2, tshift):
    
       sizex, sizey, size_tminus, size_tplus = crop_size
       
       imagesizex = sizex * gridx
       imagesizey = sizey * gridy
       
       shiftNone = [0,0]
       AllShifts = [shiftNone]


       time = time - tshift
       if time > 0:
               for shift in AllShifts:

                        newname = name + 'shift' + str(shift)
                        Event_data = []
                        
                        if yolo_v1:    
                            Label = np.zeros([total_categories + 6])
                        if yolo_v2:
                            Label = np.zeros([total_categories + 7])
                        Label[trainlabel] = 1
                        
                        newcenter = (y - shift[1],x - shift[0] )
                        if x + shift[0]> sizex/2 and y + shift[1] > sizey/2 and x + shift[0] + int(imagesizex/2) < image.shape[2] and y + shift[1]+ int(imagesizey/2) < image.shape[1] and time > size_tminus and time + size_tplus + 1 < image.shape[0]:
                                crop_xminus = x  - int(imagesizex/2)
                                crop_xplus = x  + int(imagesizex/2)
                                crop_yminus = y  - int(imagesizey/2)
                                crop_yplus = y   + int(imagesizey/2)
                                # Cut off the region for training movie creation
                                region =(slice(int(time - size_tminus),int(time + size_tplus  + 1)),slice(int(crop_yminus)+ shift[1], int(crop_yplus)+ shift[1]),
                                      slice(int(crop_xminus) + shift[0], int(crop_xplus) + shift[0]))
                                #Define the movie region volume that was cut
                                crop_image = image[region]   
                                if normPatch:
                                    crop_image = normalizeFloatZeroOne( crop_image.astype('uint16'),1,99.8)
                                seglocationx = (newcenter[1] - crop_xminus)
                                seglocationy = (newcenter[0] - crop_yminus)
                                Label[total_categories] =  seglocationx/sizex
                                Label[total_categories + 1] = seglocationy/sizey
                                Label[total_categories + 2] = (size_tminus) / (size_tminus + size_tplus)             
                                Label[total_categories + 3] = 1
                                Label[total_categories + 4] = 1
                                Label[total_categories + 5] = 1
       

                                #Write the image as 32 bit tif file 
                                if(crop_image.shape[0] == size_tplus + size_tminus + 1 and crop_image.shape[1]== imagesizey and crop_image.shape[2]== imagesizex):

                                           imwrite((save_dir + '/' + newname + '.tif'  ) , crop_image.astype('float32'))    
                                           Event_data.append([Label[i] for i in range(0,len(Label))])
                                           if(os.path.exists(save_dir + '/' + (newname) + ".csv")):
                                                        os.remove(save_dir + '/' + (newname) + ".csv")
                                           writer = csv.writer(open(save_dir + '/' + (newname) + ".csv", "a"))
                                           writer.writerows(Event_data)

def SimpleMovieMaker4D(normalizeimage, time, z, y, x, image, crop_size, gridx, gridy, total_categories, trainlabel, name, save_dir, normPatch, yolo_v1, yolo_v2, tshift):
    
       sizex, sizey, size_tminus, size_tplus = crop_size
       
       imagesizex = sizex * gridx
       imagesizey = sizey * gridy
       
       shiftNone = [0,0]
       AllShifts = [shiftNone]

       
       time = time - tshift

       image = image[:,z,:,:]
       if normalizeimage:
                    image = normalizeFloatZeroOne( image.astype('uint16'),1,99.8)

       if time > 0:
               
               for shift in AllShifts:

                        newname = name + 'shift' + str(shift)
                        Event_data = []
                        
                        
                        if yolo_v1:    
                            Label = np.zeros([total_categories + 6])
                        if yolo_v2:
                            Label = np.zeros([total_categories + 7])
                        Label[trainlabel] = 1
                        
                        newcenter = (y - shift[1],x - shift[0] )
                        if x + shift[0]> sizex/2 and y + shift[1] > sizey/2 and x + shift[0] + int(imagesizex/2) < image.shape[2] and y + shift[1]+ int(imagesizey/2) < image.shape[1] and time > size_tminus and time + size_tplus + 1 < image.shape[0]:
                                crop_xminus = x  - int(imagesizex/2)
                                crop_xplus = x  + int(imagesizex/2)
                                crop_yminus = y  - int(imagesizey/2)
                                crop_yplus = y   + int(imagesizey/2)
                                # Cut off the region for training movie creation
                                region =(slice(int(time - size_tminus),int(time + size_tplus  + 1)),slice(int(crop_yminus)+ shift[1], int(crop_yplus)+ shift[1]),
                                      slice(int(crop_xminus) + shift[0], int(crop_xplus) + shift[0]))
                                #Define the movie region volume that was cut
                                crop_image = image[region]   
                                if normPatch:
                                    crop_image = normalizeFloatZeroOne( crop_image.astype('uint16'),1,99.8)

                                seglocationx = (newcenter[1] - crop_xminus)
                                seglocationy = (newcenter[0] - crop_yminus)
                                Label[total_categories] =  seglocationx/sizex
                                Label[total_categories + 1] = seglocationy/sizey
                                Label[total_categories + 2] = (size_tminus) / (size_tminus + size_tplus)      
                                Label[total_categories + 3] = 1
                                Label[total_categories + 4] = 1
                                Label[total_categories + 5] = 1
       

                                #Write the image as 32 bit tif file 
                                if(crop_image.shape[0] == size_tplus + size_tminus + 1 and crop_image.shape[1]== imagesizey and crop_image.shape[2]== imagesizex):

                                           imwrite((save_dir + '/' + newname + '.tif'  ) , crop_image.astype('float32'))    
                                           Event_data.append([Label[i] for i in range(0,len(Label))])
                                           if(os.path.exists(save_dir + '/' + (newname) + ".csv")):
                                                        os.remove(save_dir + '/' + (newname) + ".csv")
                                           writer = csv.writer(open(save_dir + '/' + (newname) + ".csv", "a"))
                                           writer.writerows(Event_data)                                                                    
    
def loadResizeImgs(im, size):
    
          
            w,h = im.size
            if w < h:
                im = im.crop((0, (h-w)/2, w, (h+w)/2))
            elif w > h:
                im = im.crop(((w-h+1)/2,0, (w+h)/2, h))

            return np.array(im.resize(size, Image.BILINEAR))
            

def Folder_to_oneat(dir, trainlabel, trainname, total_categories, size, save_dir):

        Label = np.zeros([total_categories]) 
        
        count = 0
        
        files = sorted(glob.glob(dir + "/" + '*.png'))
        for i in tqdm(range(len(files))):
                file = files[i]
                try:
                    Event_data = [] 
                   
                    img = Image.open(file)
                    img = loadResizeImgs(img, size)
                    Name = str(trainname) + os.path.basename(os.path.splitext(file)[0])
                    image = normalizeFloatZeroOne( img.astype('uint16'),1,99.8)
                    Label[trainlabel] = 1
                    imwrite((save_dir + '/' + Name + str(count)  + '.tif'  ) , image.astype('uint16'))  
                    Event_data.append([Label[i] for i in range(0,len(Label))])
                    if(os.path.exists(save_dir + '/' + Name + str(count) + ".csv")):
                        os.remove(save_dir + '/' + Name + str(count) + ".csv")
                    writer = csv.writer(open(save_dir + '/' + Name + str(count) + ".csv", "a"))
                    writer.writerows(Event_data)
                    count = count + 1
                except Exception as e:
                    print("[WW] ", str(e))
                    continue
           
            
            

def Midog_to_oneat(midog_folder, annotation_file,event_type_name_label, all_ids, crop_size, save_dir):

    rows = []
    annotations = {}
    id_to_tumortype = {id:list(k for k in all_ids if id in all_ids[k])[0] for id in range(1,406)}
    with open(annotation_file) as f:
        data = json.load(f)
    
        #categories = {cat["id"]: cat["name"] for cat in data["categories"]}
        categories = {1: 'mitotic figure', 2: 'hard negative'}
        total_categories = len(event_type_name_label.keys())
        for row in data["images"]:
            file_name = row["file_name"]
            image_id = row["id"]
            width = row["width"]
            height = row["height"]
            tumortype = id_to_tumortype[image_id]
            
            for annotation in [anno for anno in data['annotations'] if anno["image_id"] == image_id]:
                box = annotation["bbox"]
                cat = annotation["category_id"]

                rows.append([image_id, width, height, box, cat, tumortype])
            annotations[file_name] = rows
    


    
    count = 0 
    for tumortype, ids in zip(list(all_ids.keys()), list(all_ids.values())):

        for image_id in ids:

            
            
            file_path = midog_folder + "/" +  f"{image_id:03d}.tiff"
            Name = os.path.basename(os.path.splitext(file_path)[0])
           
            img = imread(file_path)
            image = normalizeFloatZeroOne( img.astype('uint16'),1,99.8)
            image_annotation_array = annotations[Name + '.tiff']
            image_id += 1

    for image_annotation in image_annotation_array:
        
        Label = np.zeros([total_categories + 5]) 
        Event_data = []
        image_id, image_width, image_height, box, cat, tumortype = image_annotation
        Name = str(image_id)
        x0, y0, x1, y1 = box
        height = y1 - y0
        width = x1 - x0
        x = (x0 + x1) //2
        y = (y0 + y1) //2
        # if cat == 1 then it is mitosis if cat == 2 it is hard negative
        if cat == 2:
            trainlabel = event_type_name_label[tumortype] + total_categories//2  
        if cat == 1 :
            trainlabel = event_type_name_label[tumortype]
        ImagesizeX, ImagesizeY = crop_size
        crop_Xminus = x  - int(ImagesizeX/2)
        crop_Xplus = x   + int(ImagesizeX/2)
        crop_Yminus = y  - int(ImagesizeY/2)
        crop_Yplus = y   + int(ImagesizeY/2)
        region =(slice(int(crop_Yminus), int(crop_Yplus)),
                                                    slice(int(crop_Xminus), int(crop_Xplus)))

        crop_image = image[region]      

        
        
        Label[trainlabel] = 1
        Label[total_categories] =  0.5
        Label[total_categories + 1] = 0.5
        Label[total_categories + 2] = width/ImagesizeX
        Label[total_categories + 3] = height/ImagesizeY
        Label[total_categories + 4] = 1 
        
        count = count + 1
        if(crop_image.shape[0]== ImagesizeY and crop_image.shape[1]== ImagesizeX):
                    imwrite((save_dir + '/' + Name + str(count)  + '.tif'  ) , crop_image.astype('float32'))  
                    Event_data.append([Label[i] for i in range(0,len(Label))])
                    if(os.path.exists(save_dir + '/' + Name + str(count) + ".csv")):
                        os.remove(save_dir + '/' + Name + str(count) + ".csv")
                    writer = csv.writer(open(save_dir + '/' + Name + str(count) + ".csv", "a"))
                    writer.writerows(Event_data)

def Midog_to_oneat_simple(midog_folder, annotation_file,event_type_name_label, all_ids, crop_size, save_dir):

    rows = []
    annotations = {}
    id_to_tumortype = {id:list(k for k in all_ids if id in all_ids[k])[0] for id in range(1,406)}
    with open(annotation_file) as f:
        data = json.load(f)
    
        #categories = {cat["id"]: cat["name"] for cat in data["categories"]}
        categories = {1: 'mitotic figure', 2: 'hard negative'}
        total_categories = len(event_type_name_label.keys())
        for row in data["images"]:
            file_name = row["file_name"]
            image_id = row["id"]
            width = row["width"]
            height = row["height"]
            tumortype = id_to_tumortype[image_id]
            
            for annotation in [anno for anno in data['annotations'] if anno["image_id"] == image_id]:
                box = annotation["bbox"]
                cat = annotation["category_id"]

                rows.append([image_id, width, height, box, cat, tumortype])
            annotations[file_name] = rows
    


    
    count = 0 
    for tumortype, ids in zip(list(all_ids.keys()), list(all_ids.values())):

        for image_id in ids:

            
            
            file_path = midog_folder + "/" +  f"{image_id:03d}.tiff"
            Name = os.path.basename(os.path.splitext(file_path)[0])
           
            img = imread(file_path)
            image = normalizeFloatZeroOne( img.astype('uint16'),1,99.8)
            image_annotation_array = annotations[Name + '.tiff']
            image_id += 1

    for image_annotation in image_annotation_array:
        
        Label = np.zeros([total_categories]) 
        Event_data = []
        image_id, image_width, image_height, box, cat, tumortype = image_annotation
        Name = str(image_id)
        x0, y0, x1, y1 = box
        height = y1 - y0
        width = x1 - x0
        x = (x0 + x1) //2
        y = (y0 + y1) //2
        # if cat == 1 then it is mitosis if cat == 2 it is hard negative
        if cat == 2:
            trainlabel = event_type_name_label[tumortype] + total_categories//2  
        if cat == 1 :
            trainlabel = event_type_name_label[tumortype]
        ImagesizeX, ImagesizeY = crop_size
        crop_Xminus = x  - int(ImagesizeX/2)
        crop_Xplus = x   + int(ImagesizeX/2)
        crop_Yminus = y  - int(ImagesizeY/2)
        crop_Yplus = y   + int(ImagesizeY/2)
        region =(slice(int(crop_Yminus), int(crop_Yplus)),
                                                    slice(int(crop_Xminus), int(crop_Xplus)))

        crop_image = image[region]      

        
        
        Label[trainlabel] = 1
        
        
        count = count + 1
        if(crop_image.shape[0]== ImagesizeY and crop_image.shape[1]== ImagesizeX):
                    imwrite((save_dir + '/' + Name + str(count)  + '.tif'  ) , crop_image.astype('float32'))  
                    Event_data.append([Label[i] for i in range(0,len(Label))])
                    if(os.path.exists(save_dir + '/' + Name + str(count) + ".csv")):
                        os.remove(save_dir + '/' + Name + str(count) + ".csv")
                    writer = csv.writer(open(save_dir + '/' + Name + str(count) + ".csv", "a"))
                    writer.writerows(Event_data)


def MovieLabelDataSet(image_dir, seg_image_dir, csv_dir, save_dir, static_name, static_label, csv_name_diff, crop_size, gridx = 1, gridy = 1,  
yolo_v1 = True, yolo_v2 = False,  tshift  = 0, normalizeimage = True):
    
    
            raw_path = os.path.join(image_dir, '*tif')
            Seg_path = os.path.join(seg_image_dir, '*tif')
            Csv_path = os.path.join(csv_dir, '*csv')
            files_raw = glob.glob(raw_path)
            files_raw.sort
            filesSeg = glob.glob(Seg_path)
            filesSeg.sort
            filesCsv = glob.glob(Csv_path)
            filesCsv.sort
            Path(save_dir).mkdir(exist_ok=True)
            total_categories = len(static_name)
                
            
            
            for fname in files_raw:
                  
                 name = os.path.basename(os.path.splitext(fname)[0])   
                 for Segfname in filesSeg:
                      
                      Segname = os.path.basename(os.path.splitext(Segfname)[0])
                        
                      if name == Segname:
                        
                         
                         for csvfname in filesCsv:
                                 count = 0  
                                 Csvname =  os.path.basename(os.path.splitext(csvfname)[0])
                                 for i in  range(0, len(static_name)):
                                     event_name = static_name[i]
                                     trainlabel = static_label[i]
                                     classfound = (Csvname == csv_name_diff +  event_name + name)   
                                     if classfound:
                                                    print(Csvname)
                                                    image = imread(fname).astype('uint8')
                                                    segimage = imread(Segfname).astype('uint16')
                                                    dataset = pd.read_csv(csvfname)
                                                    time = dataset[dataset.keys()[0]][1:]
                                                    y = dataset[dataset.keys()[1]][1:]
                                                    x = dataset[dataset.keys()[2]][1:]
                                                                            
                                                    try:
                                                       angle = dataset[dataset.keys()[4]][1:]
                                                    except:   
                                                       angle = dataset[dataset.keys()[0]][1:] * 0 + 2    

                                                    #Categories + XYHW + Confidence 
                                                    for (key, t) in time.items():
                                                      
                                                          MovieMaker(t, y[key], x[key], angle[key], image, segimage, 
                                                          crop_size, gridx, gridy, total_categories, trainlabel, 
                                                          name + event_name + str(count), save_dir, yolo_v1, yolo_v2, tshift, normalizeimage)
                                                          count = count + 1
                                                    image = None
                                                    segimage = None      
                                                        
                                                       
                                                        
def VolumeLabelDataSet(image_dir, seg_image_dir, csv_dir, save_dir, static_name, static_label, csv_name_diff, crop_size, gridx = 1, gridy = 1, gridz = 1,  
yolo_v1 = True, yolo_v2 = False,  tshift  = 0, normalizeimage = True):
    
    
            raw_path = os.path.join(image_dir, '*tif')
            Seg_path = os.path.join(seg_image_dir, '*tif')
            Csv_path = os.path.join(csv_dir, '*csv')
            files_raw = glob.glob(raw_path)
            files_raw.sort
            filesSeg = glob.glob(Seg_path)
            filesSeg.sort
            filesCsv = glob.glob(Csv_path)
            filesCsv.sort
            Path(save_dir).mkdir(exist_ok=True)
            total_categories = len(static_name)
            
            for fname in files_raw:
                  
                 name = os.path.basename(os.path.splitext(fname)[0])   
                 for Segfname in filesSeg:
                      
                      Segname = os.path.basename(os.path.splitext(Segfname)[0])
                        
                      if name == Segname:
                         
                         for csvfname in filesCsv:
                                 count = 0  
                                 Csvname =  os.path.basename(os.path.splitext(csvfname)[0])
                                 for i in  range(0, len(static_name)):
                                     event_name = static_name[i]
                                     trainlabel = static_label[i]
                                     classfound = (Csvname == csv_name_diff +  event_name + name)   
                                     if classfound:
                                                    print(Csvname)
                                                    image = imread(fname).astype('uint8')
                                                    segimage = imread(Segfname).astype('uint16')
                                                    dataset = pd.read_csv(csvfname)
                                                    time = dataset[dataset.keys()[0]][1:]
                                                    z = dataset[dataset.keys()[1]][1:]
                                                    y = dataset[dataset.keys()[2]][1:]
                                                    x = dataset[dataset.keys()[3]][1:]
                                                    try:
                                                       angle = dataset[dataset.keys()[4]][1:]
                                                    except:   
                                                       angle = dataset[dataset.keys()[0]][1:] * 0 + 2                      

                                                                                  
                                                    #Categories + XYZHW + Confidence 
                                                    for (key, t) in time.items():
                                                          
                                                          VolumeMaker(t, z[key], y[key], x[key], angle[key], image, segimage, 
                                                          crop_size, gridx, gridy,gridz, total_categories, trainlabel, 
                                                          name + event_name + str(count), save_dir, yolo_v1, yolo_v2, tshift, normalizeimage)
                                                          count = count + 1
                                                    image = None
                                                    segimage = None   
                                                    
                                 

               
def VolumeMaker(time, z, y, x, angle, image, segimage, crop_size, gridx, gridy,gridz,  total_categories, trainlabel, name, save_dir,  yolo_v1, yolo_v2, tshift,normalizeimage ):
    

       sizex, sizey, sizez, size_tminus, size_tplus = crop_size
       
       imagesizex = sizex * gridx
       imagesizey = sizey * gridy
       imagesizez = sizez * gridz
       shiftNone = [0,0,0]
       
       AllShifts = [shiftNone]


       time = time - tshift
       
       if time > size_tminus:

               #slice the images

               currentsegimage = segimage[int(time),:].astype('uint16')
               image_props = getHWD(x, y, z, currentsegimage, imagesizex, imagesizey,imagesizez)
               if image_props is not None:
                    height, width, depth, center, seg_label = image_props
                    smallimage = CreateVolume(image, size_tminus, size_tplus, int(time))
                    if normalizeimage:
                        smallimage = normalizeFloatZeroOne(smallimage.astype('uint16'), 1, 99.8)
                    for shift in AllShifts:

                                newname = name + 'shift' + str(shift)
                                Event_data = []
                                newcenter = center
                                x = center[2]
                                y = center[1]
                                z = center[0]
                            
                                if yolo_v1:    
                                    Label = np.zeros([total_categories + 8])
                                if yolo_v2:
                                    Label = np.zeros([total_categories + 9])
                                Label[trainlabel] = 1
                                
                                #T co ordinate
                                Label[total_categories + 3] = (size_tminus) / (size_tminus + size_tplus)
                                if x > sizex/2 and z > sizez/2 and  y  > sizey/2 and z  + int(imagesizez/2) < image.shape[1] and y  + int(imagesizey/2) < image.shape[2] and x + int(imagesizex/2) < image.shape[3] and time > size_tminus and time + size_tplus + 1 < image.shape[0]:
                                                crop_xminus = x  - int(imagesizex/2)
                                                crop_xplus = x  + int(imagesizex/2)
                                                crop_yminus = y  - int(imagesizey/2)
                                                crop_yplus = y   + int(imagesizey/2)
                                                crop_zminus = z  - int(imagesizez/2)
                                                crop_zplus = z   + int(imagesizez/2)
                                                region =(slice(0,smallimage.shape[0]),slice(int(crop_zminus), int(crop_zplus)), slice(int(crop_yminus), int(crop_yplus)),
                                                    slice(int(crop_xminus) , int(crop_xplus) ))
                                                    
                                                #Define the movie region volume that was cut
                                                crop_image = smallimage[region]
                                                seglocationx = (newcenter[2] - crop_xminus)
                                                seglocationy = (newcenter[1] - crop_yminus)
                                                seglocationz = (newcenter[0] - crop_zminus)

                                                Label[total_categories] =  seglocationx/sizex
                                                Label[total_categories + 1] = seglocationy/sizey
                                                Label[total_categories + 2] = seglocationz/sizez
                                                if height >= imagesizey:
                                                                height = 0.5 * imagesizey
                                                if width >= imagesizex:
                                                                width = 0.5 * imagesizex
                                                if depth >= imagesizez:
                                                                depth = 0.5 * imagesizez                
                                                #Height
                                                Label[total_categories + 4] = height/imagesizey
                                                #Width
                                                Label[total_categories + 5] = width/imagesizex
                                                #Depth
                                                Label[total_categories + 6] = depth/imagesizez



                                                if yolo_v1:
                                                        Label[total_categories + 7] = 1 
                                                        
                                                if yolo_v2:

                                                    Label[total_categories + 7] = 1 
                                                    Label[total_categories + 8] = angle        
                                                #Write the image as 32 bit tif file 
                                                if(crop_image.shape[0] == size_tplus + size_tminus + 1 and crop_image.shape[1]== imagesizez and crop_image.shape[2]== imagesizey and crop_image.shape[3]== imagesizex ):

                                                        imwrite((save_dir + '/' + newname + '.tif'  ) , crop_image.astype('float32'))    
                                                        Event_data.append([Label[i] for i in range(0,len(Label))])
                                                        if(os.path.exists(save_dir + '/' + (newname) + ".csv")):
                                                                        os.remove(save_dir + '/' + (newname) + ".csv")
                                                        writer = csv.writer(open(save_dir + '/' + (newname) + ".csv", "a"))
                                                        writer.writerows(Event_data)
       crop_image = None
       currentsegimage = None
       smallimage = None


            
def MovieMaker(time, y, x, angle, image, segimage, crop_size, gridx, gridy,  total_categories, trainlabel,
name, save_dir, yolo_v1, yolo_v2, tshift, normalizeimage):
    
       sizex, sizey, size_tminus, size_tplus = crop_size
       
       imagesizex = sizex * gridx
       imagesizey = sizey * gridy
       
       shiftNone = [0,0]
       AllShifts = [shiftNone]


       time = time - tshift
       if time > size_tminus:
               currentsegimage = segimage[int(time),:].astype('uint16')
               image_props = getHW(x, y, currentsegimage, imagesizex, imagesizey)
               if image_props is not None:
                    height, width, center, seg_label = image_props
                    for shift in AllShifts:

                                newname = name + 'shift' + str(shift)
                                Event_data = []
                                newcenter = (center[0] - shift[1],center[1] - shift[0] )
                                x = center[1]
                                y = center[0]
                            
                                if yolo_v1:    
                                    Label = np.zeros([total_categories + 6])
                                if yolo_v2:
                                    Label = np.zeros([total_categories + 7])
                                Label[trainlabel] = 1
                                #T co ordinate
                                Label[total_categories + 2] = (size_tminus) / (size_tminus + size_tplus)
                                smallimage = CreateVolume(image, size_tminus, size_tplus, int(time))
                                if normalizeimage:
                                    smallimage = normalizeFloatZeroOne(smallimage.astype('uint16'), 1, 99.8)
                                if x + shift[0]> sizex/2 and y + shift[1] > sizey/2 and x + shift[0] + int(imagesizex/2) < image.shape[2] and y + shift[1]+ int(imagesizey/2) < image.shape[1] and time > size_tminus and time + size_tplus + 1 < image.shape[0]:
                                                crop_xminus = x  - int(imagesizex/2)
                                                crop_xplus = x  + int(imagesizex/2)
                                                crop_yminus = y  - int(imagesizey/2)
                                                crop_yplus = y   + int(imagesizey/2)
                                                region =(slice(0,smallimage.shape[0]),slice(int(crop_yminus)+ shift[1], int(crop_yplus)+ shift[1]),
                                                    slice(int(crop_xminus) + shift[0], int(crop_xplus) + shift[0]))
                                                #Define the movie region volume that was cut
                                                crop_image = smallimage[region]   

                                                seglocationx = (newcenter[1] - crop_xminus)
                                                seglocationy = (newcenter[0] - crop_yminus)

                                                Label[total_categories] =  seglocationx/sizex
                                                Label[total_categories + 1] = seglocationy/sizey
                                                if height >= imagesizey:
                                                                height = 0.5 * imagesizey
                                                if width >= imagesizex:
                                                                width = 0.5 * imagesizex
                                                #Height
                                                Label[total_categories + 3] = height/imagesizey
                                                #Width
                                                Label[total_categories + 4] = width/imagesizex

                                                if yolo_v1:
                                                        Label[total_categories + 5] = 1 
                                                        
                                                if yolo_v2:
                                                    Label[total_categories + 5] = 1 
                                                    Label[total_categories + 6] = angle        
                                                #Write the image as 32 bit tif file 
                                                if(crop_image.shape[0] == size_tplus + size_tminus + 1 and crop_image.shape[1]== imagesizey and crop_image.shape[2]== imagesizex):

                                                        imwrite((save_dir + '/' + newname + '.tif'  ) , crop_image.astype('float32'))    
                                                        Event_data.append([Label[i] for i in range(0,len(Label))])
                                                        if(os.path.exists(save_dir + '/' + (newname) + ".csv")):
                                                                        os.remove(save_dir + '/' + (newname) + ".csv")
                                                        writer = csv.writer(open(save_dir + '/' + (newname) + ".csv", "a"))
                                                        writer.writerows(Event_data)
       image = None
       segimage = None 
       crop_image = None
       currentsegimage = None
       smallimage = None
                                                  
       



def Readname(fname):
    
    return os.path.basename(os.path.splitext(fname)[0])


def ImageLabelDataSet(image_dir, seg_image_dir, csv_dir,save_dir, static_name, static_label, csv_name_diff,crop_size, gridx = 1, gridy = 1, tshift  = 0):
    
    
            raw_path = os.path.join(image_dir, '*tif')
            Seg_path = os.path.join(seg_image_dir, '*tif')
            Csv_path = os.path.join(csv_dir, '*csv')
            files_raw = glob.glob(raw_path)
            files_raw.sort
            filesSeg = glob.glob(Seg_path)
            filesSeg.sort
            filesCsv = glob.glob(Csv_path)
            filesCsv.sort
            Path(save_dir).mkdir(exist_ok=True)
            total_categories = len(static_name)
            
            for csvfname in filesCsv:
              print(csvfname)
              count = 0
              Csvname =  os.path.basename(os.path.splitext(csvfname)[0])
            
              for fname in files_raw:
                  
                 name = os.path.basename(os.path.splitext(fname)[0])   
                 for Segfname in filesSeg:
                      
                      Segname = os.path.basename(os.path.splitext(Segfname)[0])
                        
                      if name == Segname:
                          
                          
                         
                         for i in  range(0, len(static_name)):
                             event_name = static_name[i]
                             trainlabel = static_label[i]
                             if Csvname == csv_name_diff + name + event_name:
                                            image = imread(fname)
                                            image = normalizeFloatZeroOne( image.astype('uint16'),1,99.8)   
                                            segimage = imread(Segfname)
                                            dataset = pd.read_csv(csvfname)
                                            time = dataset[dataset.keys()[0]][1:]
                                            y = dataset[dataset.keys()[1]][1:]
                                            x = dataset[dataset.keys()[2]][1:]     
                                            
                                            #Categories + XYHW + Confidence 
                                            for (key, t) in time.items():
                                               ImageMaker(t, y[key], x[key], image, segimage, crop_size, gridx, gridy, total_categories, trainlabel, name + event_name + str(count), save_dir,tshift)    
                                               count = count + 1
    


    
def SegFreeImageLabelDataSet(image_dir, csv_dir,save_dir, static_name, static_label, csv_name_diff,crop_size, gridx = 1, gridy = 1):
    
    
            raw_path = os.path.join(image_dir, '*tif')
            Csv_path = os.path.join(csv_dir, '*csv')
            files_raw = glob.glob(raw_path)
            files_raw.sort
            filesCsv = glob.glob(Csv_path)
            filesCsv.sort
            Path(save_dir).mkdir(exist_ok=True)
            total_categories = len(static_name)
            
            for csvfname in filesCsv:
              print(csvfname)
              count = 0
              Csvname =  os.path.basename(os.path.splitext(csvfname)[0])
            
              for fname in files_raw:
                  
                         name = os.path.basename(os.path.splitext(fname)[0])   
                         image = imread(fname)
                       
                         image = normalizeFloatZeroOne( image.astype('uint16'),1,99.8)   
                         for i in  range(0, len(static_name)):
                             event_name = static_name[i]
                             trainlabel = static_label[i]
                             if Csvname == csv_name_diff + name + event_name:
                                            dataset = pd.read_csv(csvfname)
                                            time = dataset[dataset.keys()[0]][1:]
                                            y = dataset[dataset.keys()[1]][1:]
                                            x = dataset[dataset.keys()[2]][1:]     
                                            
                                            #Categories + XYHW + Confidence 
                                            for (key, t) in time.items():
                                               SegFreeImageMaker(t, y[key], x[key], image, crop_size, gridx, gridy,  total_categories, trainlabel, name + event_name + str(count), save_dir)    
                                               count = count + 1                 




def CreateVolume(patch, size_tminus, size_tplus, timepoint):
    starttime = timepoint - int(size_tminus)
    endtime = timepoint + int(size_tplus) + 1
    smallimg = patch[starttime:endtime, :]

    return smallimg


def createNPZ(save_dir, axes, save_name = 'Yolov0oneat', save_name_val = 'Yolov0oneatVal', expand = True, 
static = False, flip_channel_axis = False, train_size = 0.95):
            
            data = []
            label = []   
             
            raw_path = os.path.join(save_dir, '*tif')
            files_raw = glob.glob(raw_path)
            files_raw.sort
            NormalizeImages= [imread(fname) for fname in files_raw]
            
            names = [Readname(fname)  for fname in files_raw]
            #Normalize everything before it goes inside the training
            for i in range(0,len(NormalizeImages)):
                   
                       n = NormalizeImages[i]
                   
                       blankX = n
                       csvfname = save_dir + '/' + names[i] + '.csv'   
                       arr = [] 
                       with open(csvfname) as csvfile:
                             reader = csv.reader(csvfile, delimiter = ',')
                             arr =  list(reader)[0]
                             arr = np.array(arr)
                            
                       blankY = arr
                       blankY = np.expand_dims(blankY, -1)
                       if expand:
                         
                         blankX = np.expand_dims(blankX, -1)
                     
                       data.append(blankX)
                       label.append(blankY)



            dataarr = np.asarray(data)
            labelarr = np.asarray(label)
            if flip_channel_axis:
                       np.swapaxes(dataarr, 1,-1)
            if static:
                try:
                    
                   dataarr = dataarr[:,0,:,:,:]
                except:
                    
                    pass
            print(dataarr.shape, labelarr.shape)
            traindata, validdata, trainlabel, validlabel = train_test_split(dataarr, labelarr, train_size = train_size,
            test_size = 1 - train_size, shuffle = True)
            save_full_training_data(save_dir, save_name, traindata, trainlabel, axes)
            save_full_training_data(save_dir, save_name_val, validdata, validlabel, axes)


def _raise(e):
    raise e
def  ImageMaker(time, y, x, image, segimage, crop_size, gridX, gridY, total_categories, trainlabel, name, save_dir, tshift):

               sizeX, sizeY = crop_size

               ImagesizeX = sizeX * gridX
               ImagesizeY = sizeY * gridY

               shiftNone = [0,0]
               AllShifts = [shiftNone]

               time = time - tshift
               if time < segimage.shape[0] - 1 and time > 0:
                 currentsegimage = segimage[int(time),:].astype('uint16')
                
                 image_props  = getHW(x, y, currentsegimage, ImagesizeX, ImagesizeY)
                 if image_props is not None:
                        height, width, center, seg_label = image_props
                        for shift in AllShifts:
                        
                                newname = name + 'shift' + str(shift)
                                newcenter = (center[0] - shift[1],center[1] - shift[0] )
                                Event_data = []
                                
                                x = center[1]
                                y = center[0]
                                Label = np.zeros([total_categories + 5])  
                                Label[trainlabel] = 1
                                if x + shift[0]> sizeX/2 and y + shift[1] > sizeY/2 and x + shift[0] + int(ImagesizeX/2) < image.shape[2] and y + shift[1]+ int(ImagesizeY/2) < image.shape[1]:
                                            crop_Xminus = x  - int(ImagesizeX/2)
                                            crop_Xplus = x   + int(ImagesizeX/2)
                                            crop_Yminus = y  - int(ImagesizeY/2)
                                            crop_Yplus = y   + int(ImagesizeY/2)
                                            
                                            for tex in range(int(time) -2, int(time) + 2):
                                                            newname = newname + str(tex)
                                                            region =(slice(int(tex - 1),int(tex)),slice(int(crop_Yminus)+ shift[1], int(crop_Yplus)+ shift[1]),
                                                                slice(int(crop_Xminus) + shift[0], int(crop_Xplus) + shift[0]))

                                                            crop_image = image[region]      


                                                            seglocationx = (newcenter[1] - crop_Xminus)
                                                            seglocationy = (newcenter[0] - crop_Yminus)

                                                            Label[total_categories] =  seglocationx/sizeX
                                                            Label[total_categories + 1] = seglocationy/sizeY

                                                            if height >= ImagesizeY:
                                                                height = 0.5 * ImagesizeY
                                                            if width >= ImagesizeX:
                                                                width = 0.5 * ImagesizeX

                                                            Label[total_categories + 2] = height/ImagesizeY
                                                            Label[total_categories + 3] = width/ImagesizeX
                                                            Label[total_categories + 4] = 1 

                                                            if(crop_image.shape[1]== ImagesizeY and crop_image.shape[2]== ImagesizeX):
                                                                    imwrite((save_dir + '/' + newname + '.tif'  ) , crop_image.astype('float32'))  
                                                                    Event_data.append([Label[i] for i in range(0,len(Label))])
                                                                    if(os.path.exists(save_dir + '/' + (newname) + ".csv")):
                                                                        os.remove(save_dir + '/' + (newname) + ".csv")
                                                                    writer = csv.writer(open(save_dir + '/' + (newname) + ".csv", "a"))
                                                                    writer.writerows(Event_data)

def  SegFreeImageMaker(time, y, x, image, crop_size, gridX, gridY, total_categories, trainlabel, name, save_dir, tshift):

               sizex, sizey = crop_size

               ImagesizeX = sizex * gridX
               ImagesizeY = sizey * gridY

               shiftNone = [0,0]
               AllShifts = [shiftNone]

               time = time - tshift
               if time < image.shape[0] - 1 and time > 0:
                
                 for shift in AllShifts:
                   
                        newname = name + 'shift' + str(shift)
                        newcenter = (y - shift[1],x - shift[0] )
                        Event_data = []
                        
                        Label = np.zeros([total_categories + 5])  
                        Label[trainlabel] = 1
                        if x + shift[0]> sizex/2 and y + shift[1] > sizey/2 and x + shift[0] + int(ImagesizeX/2) < image.shape[2] and y + shift[1]+ int(ImagesizeY/2) < image.shape[1]:
                                    crop_Xminus = x  - int(ImagesizeX/2)
                                    crop_Xplus = x   + int(ImagesizeX/2)
                                    crop_Yminus = y  - int(ImagesizeY/2)
                                    crop_Yplus = y   + int(ImagesizeY/2)
                                    
                                    for tex in range(int(time) -2, int(time) + 2):
                                                    newname = newname + str(tex)
                                                    region =(slice(int(tex - 1),int(tex)),slice(int(crop_Yminus)+ shift[1], int(crop_Yplus)+ shift[1]),
                                                           slice(int(crop_Xminus) + shift[0], int(crop_Xplus) + shift[0]))

                                                    crop_image = image[region]      


                                                    seglocationx = (newcenter[1] - crop_Xminus)
                                                    seglocationy = (newcenter[0] - crop_Yminus)

                                                    Label[total_categories] =  seglocationx/sizex
                                                    Label[total_categories + 1] = seglocationy/sizey
                                                    Label[total_categories + 2] = 1
                                                    Label[total_categories + 3] = 1
                                                    Label[total_categories + 4] = 1 

                                                    if(crop_image.shape[1]== ImagesizeY and crop_image.shape[2]== ImagesizeX):
                                                             imwrite((save_dir + '/' + newname + '.tif'  ) , crop_image.astype('float32'))  
                                                             Event_data.append([Label[i] for i in range(0,len(Label))])
                                                             if(os.path.exists(save_dir + '/' + (newname) + ".csv")):
                                                                os.remove(save_dir + '/' + (newname) + ".csv")
                                                             writer = csv.writer(open(save_dir + '/' + (newname) + ".csv", "a"))
                                                             writer.writerows(Event_data)



       
def getHW(defaultX, defaultY, currentsegimage, imagesizex, imagesizey):
    
    properties = measure.regionprops(currentsegimage)
    
    TwoDLocation = (defaultY,defaultX)
    if int(TwoDLocation[0]) < currentsegimage.shape[0] and int(TwoDLocation[1]) < currentsegimage.shape[1]:
            SegLabel = currentsegimage[int(TwoDLocation[0]), int(TwoDLocation[1])]
            for prop in properties:
                                                    
                        if SegLabel > 0 and prop.label == SegLabel:
                                            minr, minc, maxr, maxc = prop.bbox
                                            center = (defaultY, defaultX)
                                            height =  abs(maxc - minc)
                                            width =  abs(maxr - minr)
                                            return height, width, center, SegLabel
                                        
                        if SegLabel == 0 :
                            
                                    center = (defaultY, defaultX)
                                    height = 0.5 * imagesizex
                                    width = 0.5 * imagesizey
                                    return height, width, center, SegLabel
                               
                    
                                
         


def getHWD(defaultX, defaultY, defaultZ, currentsegimage, imagesizex, imagesizey, imagesizez):
    
    properties = measure.regionprops(currentsegimage)
    DLocation = (defaultZ,defaultY,defaultX)
    if int(DLocation[0]) < currentsegimage.shape[0] and int(DLocation[1]) < currentsegimage.shape[1] and int(DLocation[2]) < currentsegimage.shape[2]:
            SegLabel = currentsegimage[int(DLocation[0]), int(DLocation[1]),int(DLocation[2]) ]
            for prop in properties:
                                                    
                        if SegLabel > 0 and prop.label == SegLabel:
                                            minr, minc,mind, maxr, maxc, maxd = prop.bbox
                                            center = (defaultZ, defaultY, defaultX)
                                            height =  abs(maxc - minc)
                                            width =  abs(maxr - minr)
                                            depth = abs(maxd - mind)
                                            return height, width, depth, center, SegLabel
                                        
                        if SegLabel == 0 :
                            
                                    center = (defaultZ, defaultY, defaultX)
                                    height = 0.5 * imagesizex
                                    width = 0.5 * imagesizey
                                    depth = 0.5 * imagesizez
                                    return height, width, depth, center, SegLabel
                               
                    
                                
            

def save_full_training_data(directory, filename, data, label, axes):
    """Save training data in ``.npz`` format."""
  

    len(axes) == data.ndim or _raise(ValueError())
    np.savez(directory + filename, data = data, label = label, axes = axes)
    
def InterchangeTXY(TXYCSV, save_dir):

    
     dataset = pd.read_csv(TXYCSV)
     time = dataset[dataset.keys()[0]][1:]
     x = dataset[dataset.keys()[1]][1:]
     y = dataset[dataset.keys()[2]][1:]
 
     Event_data = []
     
     Name = os.path.basename(os.path.splitext(TXYCSV)[0])

     for (key, t) in time.items():
         
         Event_data.append([t, y[key], x[key]])
         
     writer = csv.writer(open(save_dir + '/' + (Name) + ".csv", "a"))
     writer.writerows(Event_data)    
         
    
def  AngleAppender(AngleCSV, ONTCSV, save_dir, ColumnA = 'Y'):

     dataset = pd.read_csv(AngleCSV)
     time = dataset[dataset.keys()[0]][1:]
     if ColumnA == 'Y':
       y = dataset[dataset.keys()[1]][1:]
       x = dataset[dataset.keys()[2]][1:]
       angle = dataset[dataset.keys()[3]][1:]
     else:
       x = dataset[dataset.keys()[1]][1:]
       y = dataset[dataset.keys()[2]][1:]
       angle = dataset[dataset.keys()[3]][1:]  
       
     clickeddataset = pd.read_csv(ONTCSV)  
     
     clickedtime = clickeddataset[clickeddataset.keys()[0]][1:]
     clickedy = clickeddataset[clickeddataset.keys()[1]][1:]
     clickedx = clickeddataset[clickeddataset.keys()[2]][1:]
                              
     Event_data = []
     
     Name = os.path.basename(os.path.splitext(ONTCSV)[0])
     

     
     for (clickedkey, clickedt) in clickedtime.items():
                
                       
                for (key, t) in time.items(): 
                         
                          if t == clickedt and y[key] == clickedy[clickedkey] and x[key] == clickedx[clickedkey]:
                              
                                Event_data.append([t,y[key],x[key],angle[key]])
                              
                                     
                              
                               
     writer = csv.writer(open(save_dir + '/' + (Name) + ".csv", "a"))
     writer.writerows(Event_data)
         



# -*- coding: utf-8 -*-
"""
Created on Tue Dec 21 16:54:50 2021

@author: stardist devs
"""

# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals, absolute_import, division
import os, shutil
from collections import OrderedDict
from warnings import warn
from csbdeep.utils import _raise
from csbdeep.utils.six import Path
from oneat.NEATUtils.helpers import load_json
from csbdeep.utils.tf import keras_import
get_file = keras_import('utils', 'get_file')


_MODELS = {}
_ALIASES = {}


def clear_models_and_aliases(*cls):
    if len(cls) == 0:
        _MODELS.clear()
        _ALIASES.clear()
    else:
        for c in cls:
            if c in _MODELS:  del _MODELS[c]
            if c in _ALIASES: del _ALIASES[c]


def register_model(cls, key, url, hash, cordkey, cordurl, cordhash, catkey, caturl, cathash, paramkey, paramurl, paramhash):
    # key must be a valid file/folder name in the file system
    models = _MODELS.setdefault(cls,OrderedDict())
    key not in models or warn("re-registering model '%s' (was already registered for '%s')" % (key, cls.__name__))
    models[key] = dict(url=url, hash=hash, cordkey = cordkey, cordurl = cordurl, cordhash = cordhash, catkey = catkey, caturl = caturl, cathash = cathash, paramkey = paramkey, paramurl = paramurl, paramhash = paramhash)
    


def register_aliases(cls, key, *names):
    # aliases can be arbitrary strings
    if len(names) == 0: return
    models = _MODELS.get(cls,{})
    key in models or _raise(ValueError("model '%s' is not registered for '%s'" % (key, cls.__name__)))
    aliases = _ALIASES.setdefault(cls,OrderedDict())
    for name in names:
        aliases.get(name,key) == key or warn("alias '%s' was previously registered with model '%s' for '%s'" % (name, aliases[name], cls.__name__))
        aliases[name] = key



def get_registered_models(cls, return_aliases=True, verbose=False):
    models = _MODELS.get(cls,{})
    aliases = _ALIASES.get(cls,{})
    model_keys = tuple(models.keys())
    model_aliases = {key: tuple(name for name in aliases if aliases[name] == key) for key in models}
    if verbose:
        # this code is very messy and should be refactored...
        _n = len(models)
        _str_model  = 'model' if _n == 1 else 'models'
        _str_is_are = 'is' if _n == 1 else 'are'
        _str_colon  = ':' if _n > 0 else ''
        print("There {is_are} {n} registered {model_s} for '{clazz}'{c}".format(
              n=_n, clazz=cls.__name__, is_are=_str_is_are, model_s=_str_model, c=_str_colon))
        if _n > 0:
            print()
            _maxkeylen = 2 + max(len(key) for key in models)
            print("Name{s}Alias(es)".format(s=' '*(_maxkeylen-4+3)))
            print("────{s}─────────".format(s=' '*(_maxkeylen-4+3)))
            for key in models:
                _aliases = '   '
                _m = len(model_aliases[key])
                if _m > 0:
                    _aliases += "'%s'" % "', '".join(model_aliases[key])
                else:
                    _aliases += "None"
                _key = ("{s:%d}"%_maxkeylen).format(s="'%s'"%key)
                print("{key}{aliases}".format(key=_key, aliases=_aliases))
    return ((model_keys, model_aliases) if return_aliases else model_keys)


def get_model_details(cls, key_or_alias, verbose=False):
    models = _MODELS.get(cls,{})
    if key_or_alias in models:
        key = key_or_alias
        alias = None
    else:
        aliases = _ALIASES.get(cls,{})
        alias = key_or_alias
        alias in aliases or _raise(ValueError("'%s' is neither a key or alias for '%s'" % (alias, cls.__name__)))
        key = aliases[alias]
    if verbose:
        print("Found model '{model}'{alias_str} for '{clazz}'.".format(
        model=key, clazz=cls.__name__, alias_str=('' if alias is None else " with alias '%s'" % alias)))
    return key, alias, models[key]


def get_model_folder(cls, key_or_alias, target):
    key, alias, m = get_model_details(cls, key_or_alias)
   
    model_name = Path(get_file(fname=key+'.h5', origin=m['url'], file_hash=m['hash'],
                         cache_subdir=target, extract=True))
    cord = load_json(get_file(fname=m['cordkey']+'.json', origin=m['cordurl'], file_hash=m['cordhash'],
                         cache_subdir=target, extract=True))
    cat = load_json(get_file(fname=m['catkey']+'.json', origin=m['caturl'], file_hash=m['cathash'],
                         cache_subdir=target, extract=True))
    param = load_json(get_file(fname=m['paramkey']+'.json', origin=m['paramurl'], file_hash=m['paramhash'],
                         cache_subdir=target, extract=True))                                                   

    return model_name.stem, cord, cat, param


def get_model_instance(cls, key_or_alias, target):
    model_name, cord, cat, param = get_model_folder(cls, key_or_alias, target)
  
    model = cls(config=None, model_dir = target , model_name = os.path.splitext(model_name)[0], catconfig = cat, cordconfig = cord)
 
    return model



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Dec 23 19:12:49 2020

@author: kapoorlab
"""

#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Jul  7 15:25:10 2020

@author: Varun Kapoor
"""
import argparse

class MidogConfig(argparse.Namespace):
    
    def __init__(self, cell_position_name, cell_position_label,  **kwargs):
        
        
           self.cell_position_name = cell_position_name
           self.cell_position_label = cell_position_label
           assert len(cell_position_name) == len(cell_position_label)
    

    def to_json(self):

        
         configCord = {}
       
         for i in range(0, len(self.cell_position_name)):
            
             configCord[self.cell_position_name[i]] = self.cell_position_label[i]
        
         return configCord
         
         
        
          
   

        



import setuptools
from setuptools import find_packages, setup
from os import path

_dir = path.dirname(__file__)


with open('README.md') as f:
    long_description = f.read()
with open(path.join(_dir,'oneat','_version.py'), encoding="utf-8") as f:
    exec(f.read())

setup(
    name="oneat",

    version=__version__,

    author='Varun Kapoor',
    author_email='randomaccessiblekapoor@gmail.com',
    url='https://github.com/Kapoorlabs-CAPED/CAPED-AI-oneat/',
    description='Static and Dynamic classification tool.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    install_requires=[
        
        "pandas",
        "vollseg",
        "vollseg-napari",
        "scipy",
        "tifffile",
        "matplotlib",
        "imagecodecs",
        "napari[all]",
        "opencv-python",
        "diplib",
        "sklearn",
        "dask",
        "natsort",
        "napari-animation",
        "pydot",
        "albumentations", 
       
    ],
    packages=setuptools.find_packages(),
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.9',
    ],
)
